/*
 * Device Driver for DataTranslation DT2801
 *
 * $Id: dt2801_dio.c,v 1.2 1996/03/13 09:38:25 martin Exp $
 * $Source: /home/martin/Software/DAQ/drivers/dt2801/RCS/dt2801_dio.c,v $
 *
 *
 * $Log: dt2801_dio.c,v $
 * Revision 1.2  1996/03/13 09:38:25  martin
 * Disabeled for the moment
 *
 * Revision 1.1  1996/03/04 07:21:11  martin
 * Initial revision
 *
 *
 */

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/major.h>
#include <linux/malloc.h>
#include <linux/timer.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/dma.h>

#ifdef MODULE
#include <linux/module.h>
#include <linux/version.h>
#else
#error "dt2801_dio must be compiled as a module"
#endif

#include "dt2801_dio.h"
#include "dt2801.h"



static int dt2801_dio_read(struct inode *inode, struct file *file,
                       char* buffer, int count)
{
    return 0;
}

static int dt2801_dio_write(struct inode *inode, struct file *file,
                       char* buffer, int count)
{
    int stat;

    int minor = MINOR(inode->i_rdev);
    int port = minor & 0x3;
    int port_size = 0;


    int databyte = 0x00;

    dprintk(( "dt2801_dio_write: minor = %d  port = %d.\n", minor, port ));


    switch (port)
    {
        case 0:
        case 1:
             port_size = 1;
             if (count!=port_size) return -EINVAL;
             if ((stat = verify_area(VERIFY_READ, buffer, port_size)) != 0)
                 return stat;
             databyte = (int) get_fs_byte(buffer);
             dprintk(( "dt2801_dio_write: databyte = %d.\n", databyte ));

             if (dt2801->dioports[port].mode != DT2801_DIO_WRITE)
             {
                 dprintk(( "dt2801_dio_write: set mode to write.\n" ));
                 dt2801_writecommand(DT_C_SET_DIGOUT);
                 dt2801_writedata(port);
                 dt2801->dioports[port].mode = DT2801_DIO_WRITE;
                 dt2801->dioports[2].mode = DT2801_DIO_UNDEFINED;
             }

             dt2801_writecommand(DT_C_WRITE_DIG);
             dt2801_writedata(port);
             dt2801_writedata(databyte);

            break;

        case 2:
             port_size = 2;
             if (count!=port_size) return -EINVAL;
             if ((stat = verify_area(VERIFY_READ, buffer, port_size)) != 0)
                 return stat;
             memcpy_fromfs(&databyte, buffer, port_size);
             dprintk(( "dt2801_dio_write: databyte = %d.\n", databyte ));

             if (dt2801->dioports[port].mode != DT2801_DIO_WRITE)
             {
                 dprintk(( "dt2801_dio_write: set mode to write.\n" ));
                 dt2801_writecommand(DT_C_SET_DIGOUT);
                 dt2801_writedata(port);
                 dt2801->dioports[port].mode = DT2801_DIO_WRITE;
                 dt2801->dioports[0].mode = DT2801_DIO_WRITE;
                 dt2801->dioports[1].mode = DT2801_DIO_WRITE;
             }

             dt2801_writecommand(DT_C_WRITE_DIG);
             dt2801_writedata(port);
             dt2801_writedata2(databyte);

            break;

        default:
            dprintk(( "dt2801_dio_write: illegal port = %d.\n", port ));
            return -EINVAL;
    }

    return count;
}

static int dt2801_dio_ioctl(struct inode *inode, struct file *file,
                       unsigned int cmd, unsigned long arg)
{
    return dt2801_ioctl(inode, file, cmd, arg);
}

int dt2801_dio_cleanup(void)
{
    kfree(dt2801->dioports);

    return 0;
}

int dt2801_dio_init()
{
    dt2801->num_dios = DT2801_NUM_DIOS;
    if ( (dt2801->dioports = (Dt2801_DIO_t *)
             kmalloc( sizeof( Dt2801_DIO_t) * (dt2801->num_dios + 1), GFP_KERNEL )) == NULL)
          return -(ENOMEM);

    {
        int i;

        for (i=0; i<dt2801->num_dios+1; i++)
        {
            dt2801->dioports[i].mode = DT2801_DIO_UNDEFINED;
            dt2801->dioports[i].inuse = 0;
        }
        dt2801->dioports_sem = 0;
    }

    return 0;
}

int dt2801_dio_open(struct inode *inode, struct file *file)
{
    int minor = MINOR(inode->i_rdev);
    int port = minor & 0x3;
    Dt2801_DIO_t *dio;

    dprintk(( "dt2801_dio:open minor = % d  port = %d.\n", minor, port ));

    dio = &dt2801->dioports[port];
    if (dio->inuse)
    {
        dprintk(( "dt2801_dio:open device busy.\n" ));
        return -(EBUSY);
    }

    if (!dt2801->dioports_sem)
    {
        dprintk(( "dt2801_dio:open stop/reset.\n" ));
        dt2801_stop();
        dt2801_reset();
    }

    dio->inuse = 1;
    dt2801->dioports_sem++;
    dt2801->inuse = 1;

    return 0;
}

static void dt2801_dio_release(struct inode *inode, struct file *file)
{
    int minor = MINOR(inode->i_rdev);
    int port = minor & 0x3;
    Dt2801_DIO_t *dio;


    dprintk(( "dt2801_dio_release: minor = % d  port = %d.\n", minor, port ));

    dio = &dt2801->dioports[port];
    dio->inuse = 0;
    dt2801->dioports_sem--;

    if (!dt2801->dioports_sem) 
    {
        dt2801_release(inode, file);
    }
}

struct file_operations dt2801_dio_fops = {
    NULL,                /* seek */
    dt2801_dio_read,     /* read */
    dt2801_dio_write,    /* write */
    NULL,                /* readdir */
    NULL,                /* select */
    dt2801_dio_ioctl,    /* ioctl */
    NULL,                /* mmap */
    dt2801_open,         /* open */
    dt2801_dio_release   /* close */
};
